#include "plg.h"

#define WINSIZE_X 12.0 /* length in inches of the hosizontal of the square window */
#define WINSIZE_Y 9.0 /* length in inches of the vertical of the window */
#define NTURNS 3    /* number of turns in a game (before bonuses) */
#define RACKSIZE 4  /* number of objects in the initial level.  Increases by 1 each level. */
#define MAXSHOT 6   /* number of player shots allowed on screen at one time.. */
#define KILLERCYC 250  /* every n screen cycles a killer appears.. at least in the beginning! */
#define KILLERSPEED 0.08 /* speed at which killer moves across screen.. */
#define BONUSSCORE 10000  /* score at which a bonus turn is earned */
#define BIGROCK	0.6      /* radius of biggest rocks  */
#define ROCKSIZE_INCREMENT 0.2 /* medium rocks are this much less than biggest rocks, etc. */
#define MAXBALL 40    /* max number of 'rocks' that can be displayed */
#define MARGIN 0.5     /* the logical playing area goes this far beyond the window edge.. */
#define PENCOLOR "green"
#define BACKCOLOR "black"

#define TXT( s )  { h-=Ecurtextheight; Emov( 0.2, h ); Etext( s ); } 

#define NOSELEN 0.1
#define NOSETHETA 0.5
#define Ecircle( cx, cy, r, color, outline ) 	PLG_ellipse( cx, cy, r, r, color, outline )


/* window boundaries */
double winsize_x, winsize_y;

/* objects */
double x[MAXBALL], y[MAXBALL], oldx[MAXBALL], oldy[MAXBALL], dx[MAXBALL], dy[MAXBALL];
double size[MAXBALL];
int active[MAXBALL];
int nballs;   /* number of balls allocated.. */
int nactb;    /* number of balls visible.. */
char shapelist[15][6] = { "sym1a", "sym1a","sym1a", "sym6o", "sym4a", "sym6a", "sym2a", "sym2o", "sym5a", 
				"sym1a", "sym5o", "sym3a", "sym2a" };

char shape[8];

char colorlist[9][20] = { "green", "green", "yellow", "red", "rgb(0,1,1)", "orange", "pink", "brightblue", "magenta" };

/* ship */
static double sx[8] = { 0.200, 0.141, 0.000, -0.141, -0.200, -0.141,  0.000,  0.141 };
static double sy[8] = { 0.000, 0.141, 0.200,  0.141,  0.000, -0.141, -0.200, -0.141 };
int si;
double sposx, sposy;
double sdx, sdy;
int shipactive;

/* shots */
double shx[MAXSHOT], shy[MAXSHOT];
double oldshx[MAXSHOT], oldshy[MAXSHOT];
double shdx[MAXSHOT], shdy[MAXSHOT];
int nsh;
int shlife[MAXSHOT];


/* killers */
double kx, ky;
double kdx, kdy;
int kactive;
double ksize;
char kchar[10];
int killercyc = KILLERCYC;
double killerspeed = KILLERSPEED;

/* killer shots */
double kshx[MAXSHOT], kshy[MAXSHOT];
double oldkshx[MAXSHOT], oldkshy[MAXSHOT];
double kshdx[MAXSHOT], kshdy[MAXSHOT];
int nksh;
int kshlife[MAXSHOT];

/* misc */
unsigned int dragfactor;
long f;
double fabs();
double GL_rand();
int nrounds;
int score;
int ship;
long ifr;

int sbig, smedium, ssmall;
int stat;
int shields;
int immune;

int nturns = NTURNS;
int bonus = 0;

/* explosion sequences */
int expfrm[8] = { 0, 0, 0, 0, 0, 0, 0, 0 }; 
double expx[8], expy[8];

/* for screen-saver */
int automode;

char Pencolor[40] = PENCOLOR;
char Backcolor[40] = BACKCOLOR;


/* =============================== */
/* =============================== */
/* =============================== */
main( argc, argv )
int argc;
char *argv[];
{
register i, j, k, ie;
double h;
char s[80];


/* process args.. */
automode = 0;
winsize_x = WINSIZE_X; winsize_y = WINSIZE_Y; 
for( i = 1; i < argc; i++ ) {
	if( strcmp( argv[i], "-winsize" )==0 ) {
		i++;
		if( i < argc ) sscanf( argv[i], "%lf,%lf", &winsize_x, &winsize_y );
		}
	if( strcmp( argv[i], "-automode" )==0 ) automode = 1;
	}

PLG_set_early_defaults();
	

/* initialize */
getgui_init( "Hacksteroids", winsize_x, winsize_y, 100, 0 );


dragfactor = 75000;
nrounds = 1;
score = 0;
ship = 0;
sbig = 20;  smedium = 40; ssmall = 60;
/* Ecolor( Pencolor ); */
/* Ebackcolor( Backcolor ); */
Eclr();
Elinetype( 0, 1.5, 1.0 );


if( !automode ) {
	Eclr();
	/* Ebackcolor( "gray(0.8)" ); */
	/* Ecolor( "black" ); */
 	
	getgui_buttons( "Welcome to Hacksteroids, \n\
an X11 rip-off of the popular 70's arcade game.  \n\
\n\
Written by Steve Grubb \n\
\n\
During play, press the SPACE BAR to pause. \n\
\n\
Do you wish to adjust settings or get help?", "Yes,No", s );
	if( strcmp( s, "Yes" )==0 ) help();
	getgui_progress( "Bring your spaceship\ninto play by pressing\nthe space bar..", "" );
	sleep(2);
	getgui_progressclear();
	Ebackcolor( Backcolor );
	Ecolor( Pencolor );
	Eclr();
	}
if( automode ) {
	Emessage( "" );
	killercyc = 100;
	killerspeed = 0.2;
	}

startship();
startround();

TOP: EEvent = 0;


ifr = 0;
while( 1 ) {

	if( shipactive || automode )ifr++;
	if( EEvent != 0 ) checkbuttons();

	/* update display.. */

	/* ship */
	if( shipactive ) {
		Ecolor( Backcolor );
		Earrow( sposx, sposy, sposx+sx[si], sposy+sy[si], NOSELEN, NOSETHETA, Backcolor );
		if( immune ) Emark( sposx, sposy, "sym6a", 0.25 ); /* circle */
		sposx += sdx;
		sposy += sdy;
		checkwrap( &sposx, &sposy );

		/* update ship drag */
		if( fabs(sdx) > 0.01 ) sdx *= 0.995;
		if( fabs(sdy) > 0.01 ) sdy *= 0.995;
		Ecolor( Pencolor );
		Earrow( sposx, sposy, sposx+sx[si], sposy+sy[si], NOSELEN, NOSETHETA, Pencolor );
		if( immune ) Emark( sposx, sposy, "sym6a", 0.25 ); /* circle */

		/* see if ship hit an object.. */
		for( k = 0; k < nballs; k++ ) {
			if( ! active[k] ) continue;
			if( checkshiphit( x[k], y[k], size[k]*0.85 ) ) break;
			}
		}

	/* is it time to create a killer? */
	if( ifr % killercyc == (killercyc-1) && !kactive ) {
		kactive = 1;
		nksh = 0;
		if( GL_rand() < 0.5 ) { kx = 0.0; kdx = killerspeed; }
		else { kx = winsize_x; kdx = -killerspeed; }
		ky = GL_rand() * winsize_y;
		kdy = 0;
		if( nrounds < 2 && !automode ) strcpy( kchar, "<XXX>" );
		else  if( GL_rand() < 0.5 ) strcpy( kchar, "<XXX>" );
		else 	{
			if( GL_rand() > 0.5 ) strcpy( kchar, "<?>" );
			else strcpy( kchar, "<!>" );
			}
		if( score > 10000 && ifr > 1000 ) strcpy( kchar, "<?>" );
		if( strcmp( kchar, "<XXX>" )==0 ) ksize = 0.3;
		else if( strcmp( kchar, "<!>" )==0 ) ksize = 0.15;
		else ksize = 0.1;
		}

	/* update killer */
	if( kactive ) {
		if( ksize > 0.11 || automode ) {
			Ecolor( Backcolor );
			Emov( kx, ky ); Etext( kchar );
			}
		if( kx < -0.1 || kx > (winsize_x+MARGIN) || ky < -1 || ky > (winsize_y+MARGIN+1)) kactive = 0;
		else	{
			if( checkshiphit( kx, ky, ksize ) ) { /* killer collides with ship */

				/* start an explosion sequence */
				for( ie = 0; ie < 8; ie++ ) if( expfrm[ie] < 1 ) break;
				expx[ie] = kx; expy[ie] = ky;
				expfrm[ie] = 20;

				kactive = 0;
				}
			else if( checkrocks( kx, ky, 0 )) { /* killer collides with a rock */
				/* start an explosion sequence */
				for( ie = 0; ie < 8; ie++ ) if( expfrm[ie] < 1 ) break;
				expx[ie] = kx; expy[ie] = ky; expfrm[ie] = 20;
				kactive = 0;
				}
			else	{
				Ecolor(Pencolor);
				if( ksize < 0.2 && ky > sposy ) ky -= 0.04;
				if( ksize < 0.2 && ky < sposy ) ky += 0.04;
				if( ksize < 0.11 ) {
					kdx += ((GL_rand() * 0.04 )-0.02);
					kdy += ((GL_rand() * 0.04 )-0.02);
					}
				kx += kdx;  ky += kdy;
				Emov( kx, ky ); Etext( kchar );
				kactive++;

				if( kactive % 6 == 0 && nksh < MAXSHOT ) { /* time to fire a shot.. */
					for( k = 0; k < MAXSHOT; k++ ) if( kshlife[k] < 1 ) break;
					/* aim shot */
					if( automode || !shipactive || GL_rand() < 0.4 ) { /* random */
						kshdx[k] = ((GL_rand() * 0.8)-0.4) + kdx;
						kshdy[k] = ((GL_rand() * 0.8)-0.4) + kdy;
						}
					else if( ksize > 0.25 ) {
						kshdx[k] = (sposx - kx) / (18.0+(GL_rand()*15.0));
						kshdy[k] = (sposy - ky) / (18.0+(GL_rand()*15.0));
						}
					else if( ksize < 0.25 ) {
						kshdx[k] = (sposx - kx) / (12.0+(GL_rand()*9.0));
						kshdy[k] = (sposy - ky) / (12.0+(GL_rand()*9.0));
						}
					kshx[k] = kx + kshdx[k];
					kshy[k] = ky + kshdy[k];
					oldkshx[k] = -1; oldkshy[k] = -1;
					kshlife[k] = 20 + (int)(GL_rand() * 20.0);
					nksh++;
					}
				}
			}
		}
		
	/* animate killer shots */
	/* if( shipactive ) */ for( i = 0; i < MAXSHOT; i++ ) {
		if( kshlife[i] < 1 ) continue;
		
		Ecolor( Backcolor );
		Emov( oldkshx[i], oldkshy[i] );
		Etext( "*" );
		Ecolor(Pencolor);
		kshlife[i]--;
		if( kshlife[i] > 0 ) { 
			Emov( kshx[i], kshy[i] ); 
			Etext( "*" ); 
			oldkshx[i] = kshx[i]; oldkshy[i] = kshy[i];
			kshx[i] += kshdx[i];  kshy[i] += kshdy[i];
			}
		else nksh--;

		if( checkshiphit( kshx[i], kshy[i], 0.1 )) { /* killer shot hits ship.. */
			Ecolor( Backcolor );
			Emov( oldkshx[i], oldkshy[i] ); 
			Etext( "*" ); 
			Ecolor(Pencolor);
			kshlife[i] = 0;
			break;
			}
		stat = checkrocks( kshx[i], kshy[i], 0 ); /* see if shot hit rock, and if so, shot disappears */
		if( stat ) {
			Ecolor( Backcolor );
			Emov( oldkshx[i], oldkshy[i] ); 
			Etext( "*" ); 
			Ecolor( Pencolor );
			kshlife[i] = 0;  
			}

		if( stat == -1 ) goto TOP; /* last rock */
									
		checkwrap( &kshx[i], &kshy[i] ); /* wrap object if necessary.. */

		}

	/* animate player shots */
	if( shipactive ) {
	    for( i = 0; i < MAXSHOT; i++ ) {
		if( shlife[i] < 1 ) continue;
		
		stat = checkrocks( shx[i], shy[i], 1 ); /* see if shot hit rock, and if so, shot disappears.. */
		if( stat ) shlife[i] = 0;
		if( stat == -1 ) goto TOP; /* last rock */

		checkwrap( &shx[i], &shy[i] ); /* wrap object if necessary.. */

		if( kactive ) { /* see if shot hit a killer */
			if( fabs( shx[i] - kx ) < ksize && fabs( shy[i] - ky ) < ksize ) {
				kactive = 0;
				if( ksize > 0.25 )score += 200;
				else if( ksize > 0.11 )score += 500;
				else score += 1000;
				shlife[i] = 0;

				/* start an explosion sequence */
				for( ie = 0; ie < 8; ie++ ) if( expfrm[ie] < 1 ) break;
				expx[ie] = kx; expy[ie] = ky;
				expfrm[ie] = 20;

				Ecolor( Backcolor );
				Emov( kx, ky ); Etext( kchar );
				Ecolor(Pencolor);
				scoreboard( "[q][w]turn [o]thrust [p]fire [1]shields" );
				}
			}

		Ecolor( Backcolor );
		Emov( oldshx[i], oldshy[i] );
		Etext( "+" );
		Ecolor(Pencolor);
		shlife[i]--;
		if( shlife[i] > 0 ) { 
			Emov( shx[i], shy[i] ); 
			Etext( "+" ); 
			oldshx[i]= shx[i]; oldshy[i] = shy[i];
			shx[i] += shdx[i];  shy[i] += shdy[i];
			}
		else nsh--; 
		}
	    }

	/* objects */
	for( i = 0; i < nballs; i++ ) {
		/* if( EEvent != 0 ) checkbuttons(); */
		if( ! active[i] ) continue;
		checkwrap( &x[i], &y[i] );

		Ecolor( Backcolor );
		if( nrounds % 12 == 1 ) drawrock( oldx[i], oldy[i], size[i] );
		else if( nrounds % 12 == 2 ) drawstar( oldx[i], oldy[i], size[i] );
		else Emark( oldx[i], oldy[i], shape, size[i] );

		Ecolor(Pencolor);
		if( nrounds % 12 == 1 )drawrock( x[i], y[i], size[i] );
		else if( nrounds % 12 == 2 )drawstar( x[i], y[i], size[i] );
		else Emark( x[i], y[i], shape, size[i] ); 

		oldx[i]= x[i]; oldy[i] = y[i];
		x[i] += dx[i];  y[i] += dy[i];
		}

	/* update explosion sequences.. */
	for( i = 0; i < 8; i++ ) {
		if( expfrm[i] > 0 ) {
			h = ((expfrm[i]-7) * -1) * 0.01;
			Ecolor( Backcolor );
			expshape( expx[i], expy[i], h );
			expfrm[i]--;
			if( expfrm[i] > 0 ) {
				h = ((expfrm[i]-7) * -1) * 0.01;
				Ecolor(Pencolor);
				expshape( expx[i], expy[i], h );
				}
			}
		}
			
		
			

	/* if( nactb > 14 ) f = dragfactor; scg 12/13/99 */
	/* else f = dragfactor + (dragfactor * 0.3 * (14-nactb) );  */
	/* else f = dragfactor + (dragfactor * 0.55 * (14-nactb) ); scg 12/13/99 */
	Eflush(); /* Eusleep( f ); */  usleep( dragfactor ); 
	/* if( !automode && !shipactive ) Eusleep( 5000 ); */
	if( automode ) usleep( 20000 );
	Easync();
	}
}

/* =============================================== */

checkbuttons()
{
int k;
double h;

if( automode && EEvent == '*' ) exit(NULL);

if( EEvent == 'p' && nsh < MAXSHOT ) { /* fire shot */
	for( k = 0; k < MAXSHOT; k++ ) if( shlife[k] < 1 ) break;
	shdx[k] = sdx + sx[si]*2; 
	shdy[k] =  sdy + sy[si]*2;
	shx[k] = sposx + shdx[k];
	shy[k] = sposy + shdy[k];
	oldshx[k] = -1; oldshy[k] = -1;
	shlife[k] = 15;
	nsh++;
	if( immune ) { /* firing a shot turns off immunity */
		Ecolor( Backcolor );
		Emark( sposx, sposy, "sym6a", 0.25 ); /* circle */ 
		Ecolor( Pencolor );
		immune = 0;
		}
	}

else if( EEvent == 'q' || EEvent == 'w' ) {	/* rotate ship.. */
	/* erase old pos.. */
	Ecolor( Backcolor );
	Earrow( sposx, sposy, sposx+sx[si], sposy+sy[si], NOSELEN, NOSETHETA, Backcolor );
	Ecolor( Pencolor );
	if( EEvent == 'w' ) si--;
 	else si++; 
	if( si > 7 ) si = 0;
	if( si < 0 ) si = 7;
	if( shipactive )Earrow( sposx, sposy, sposx+sx[si], sposy+sy[si], NOSELEN, NOSETHETA, Pencolor );
	}

else if( EEvent == 'o' ) { /* thrust */
	if( fabs(sdx) < 1.5 )sdx += (sx[si]* 0.1);
	if( fabs(sdy) < 1.5 )sdy += (sy[si]* 0.1);
	}
	

else if( EEvent == '\015' ) { 
	if( shields > 0 ) { immune = 1; shields--; }
	}
	
else if( EEvent == ' ' ) { /* the space bar has several different functions, depending on when it is pressed */

	if( ship > nturns ) {  /* to exit */
		fprintf( stderr, "Final score: %d  (Bonus earned every %d)\n", score, BONUSSCORE );
		exit(0); 
		}

	else if( !shipactive ) /* to start up a new turn */
		{ shipactive = 1; EEvent = 0; return( 1 ); } 

	else help();
	}


EEvent = 0; EEventx = -1; EEventy = -1;
}


/* ====================================== */
scoreboard( str )
char str[];
{
double h;
char s[100];
if( automode ) return(1);
if( score / BONUSSCORE >= bonus+1 ) {
	bonus++;
	nturns++;
	Emessage( "BONUS EARNED!!" );
	Ecolor(Pencolor); for( h = 0.2; h < 5.0; h += 0.2 ) { Ecircle( sposx, sposy, h, -1.0, 1 ); Eflush(); }
	Ecolor(Backcolor); for( h = 0.2; h < 5.0; h += 0.2 ) { Ecircle( sposx, sposy, h, -1.0, 1 ); Eflush(); }
	Ecolor(Pencolor); for( h = 0.2; h < 5.0; h += 0.2 ) { Ecircle( sposx, sposy, h, -1.0, 1 ); Eflush(); }
	Ecolor(Backcolor); for( h = 0.2; h < 5.0; h += 0.2 ) { Ecircle( sposx, sposy, h, -1.0, 1 ); Eflush(); }
	Ecolor(Pencolor); for( h = 0.2; h < 5.0; h += 0.2 ) { Ecircle( sposx, sposy, h, -1.0, 1 ); Eflush(); }
	Ecolor(Backcolor); for( h = 0.2; h < 5.0; h += 0.2 ) { Ecircle( sposx, sposy, h, -1.0, 1 ); Eflush(); }
	Ecolor(Pencolor); for( h = 0.2; h < 5.0; h += 0.2 ) { Ecircle( sposx, sposy, h, -1.0, 1 ); Eflush(); }
	Ecolor(Backcolor); for( h = 0.2; h < 5.0; h += 0.2 ) { Ecircle( sposx, sposy, h, -1.0, 1 ); Eflush(); }
	Ecolor(Pencolor);
	killercyc -= 70;
	sbig = 30; smedium = 60; ssmall = 100;
	Eclr();
	}

sprintf( s, "%d (%d)  %d/%d  %s", score, shields, ship, nturns, str );
Emessage( s );
}

/* ====================================== */
startship()
{
int i;
if( !automode )ship++;

/* initialize ship.. */
si = 2;
sposx = winsize_x / 2.0; sposy = winsize_y / 2.0;
sdx = 0.0; sdy = 0.0;
shipactive = 0;
immune = 0;
shields = 5;

cleanup();

if( ship > nturns ) {
	char msg[100];
	sprintf( msg, "GAME OVER.  Final score: %d.  Press space bar to exit..", score );
	if( !automode )Emessage( msg );
	automode = 1;
	}
else scoreboard( "Press the space bar to begin.." );
ifr = 0; 
}


/* ====================================== */
startround()
{
int i;

strcpy( shape, shapelist[nrounds%12] );
strcpy( Pencolor, colorlist[nrounds%8] );
cleanup();

sleep(1);

/* initialize first balls */
for( i = 0; i < MAXBALL; i++ ) active[i] = 0;
if( automode ) nballs = 6;
else nballs = RACKSIZE+nrounds;
nactb = nballs;
for( i = 0; i < nballs; i++ ) {
	while( 1 ) {
		x[i] = GL_rand() * winsize_x; y[i] = GL_rand() * winsize_y;
		if( x[i] > 3.0 && x[i] < 7.0 ) continue;
		if( y[i] > 3.0 && y[i] < 7.0 ) continue;
		break;
		}
	oldx[i] = 0; oldy[i] = 0; 
	dx[i] = (GL_rand()-0.5) * 0.2; dy[i] = (GL_rand()-0.5) * 0.2; size[i] = BIGROCK;
	active[i] = 1;
	}


ifr = 0; 
}

/* ========================================== */
/* returns 1 if a rock is hit.  returns -1 if last rock is hit. */
checkrocks( inx, iny, scoreflag )
double inx, iny;
int scoreflag; /* 1 if hit is to be scored, 0 if not */
{
int i, j, k, ie, found;
double h;
char ans[10];

found = 0;
for( k = 0; k < nballs; k++ ) {
	if( ! active[k] ) continue;
	if( fabs( inx - x[k] ) < size[k] && fabs(  iny - y[k] ) < size[k] ) {

		found = 1;
		if( scoreflag ) scorerocks( size[k] );

		/* erase old ball */
		Ecolor(Backcolor);
		if( nrounds % 12 == 1 ) drawrock( oldx[k], oldy[k], size[k] );
		if( nrounds % 12 == 2 ) drawstar( oldx[k], oldy[k], size[k] );
		else Emark( oldx[k], oldy[k], shape, size[k] ); 
		Ecolor(Pencolor);

		/* start an explosion sequence */
		for( ie = 0; ie < 8; ie++ ) if( expfrm[ie] < 1 ) break;
		expx[ie] = x[k]; expy[ie] = y[k];
		if( size[k] > 0.35 ) expfrm[ie] = 6;
		else expfrm[ie] = 4;

		if( size[k] < 0.35 ) { 
			active[k] = 0; 
			nactb--;  /* vanish */
			if( nactb < 1 ) { /* last rock.. */
				nrounds++;

				/* run out last explosion sequences.. */
				for( j = 0; j < 20; j++ ) {
					usleep( 80000 );
					for( i = 0; i < 8; i++ ) {
						if( expfrm[i] > 0 ) {
							h = ((expfrm[i]-7) * -1) * 0.03;
							Ecolor(Backcolor);
							expshape( expx[i], expy[i], h );
							expfrm[i]--;
							if( expfrm[i] > 0 ) {
								h = ((expfrm[i]-7) * -1) * 0.03;
								Ecolor(Pencolor);
								expshape( expx[i], expy[i], h );
								}
							}
						}
					Eflush();
					}
				if( automode ) Eclr();

				startround();
				EEvent = 0;	
				return( -1 ); /* to indicate goto TOP */
				}
			}
		
		else	{
	
			/* make two new balls, smaller.. */
			/* 1 */
			size[k] -=  ROCKSIZE_INCREMENT;
			dx[k] += (GL_rand()-0.5) * 0.2; 
			dy[k] += (GL_rand()-0.5) * 0.2; 
			x[k] = x[k] + dx[k]; y[k] = y[k] + dy[k];
			/* 2 */
			if( nballs < (MAXBALL-1) ) {
				j = nballs;
				x[j] = x[k]; y[j] = y[k]; oldx[j] = 0; oldy[j] = 0; 
				size[j] = size[k]; active[j] = 1;
				dx[j] = dx[k]+(GL_rand()-0.5) * 0.2; 
				dy[j] = dy[k]+(GL_rand()-0.5) * 0.2;
				x[j] = x[j] + dx[j]; y[j] = y[j] + dy[j];
				nballs++; nactb++;
				}
			}

		break;
		
		}
	}
return( found );
}
/* =============================== */
checkwrap( inx, iny )
double *inx, *iny;
{
if( *inx > (winsize_x+MARGIN) ) *inx = -0.5;
if( *iny > (winsize_y+MARGIN) ) *iny = -0.5;
if( *inx < -0.5 ) *inx = (winsize_x+MARGIN);
if( *iny < -0.5 ) *iny = (winsize_y+MARGIN);
}
/* =============================== */
checkshiphit( inx, iny, insize )
double inx, iny, insize;
{
double h, g;
int found;
int ie;

found = 0;

if( ! shipactive ) return( 0 );
if( fabs( sposx - inx ) < insize && 
    fabs(  sposy - iny ) < insize ) {

	found = 1;
	if( ship == nturns ) g = 0.85;
	else g = 0.5;

	scorerocks( insize ); /* player gets points for the rock that got 'em */

	/* explosion.. */
	if( !immune ) {
		/* start an explosion sequence */
		for( ie = 0; ie < 8; ie++ ) if( expfrm[ie] < 1 ) break;
		expx[ie] = sposx; expy[ie] = sposy;
		expfrm[ie] = 20;

		Ecolor(Backcolor);
		Earrow( sposx, sposy, sposx+sx[si], sposy+sy[si], NOSELEN, NOSETHETA, Backcolor );
		Ecolor(Pencolor);
		startship();
		}
	}
return( found );
}
/* ================================ */
scorerocks( sz )
double sz;
{
/* keep score.. */
if( immune ) return(0);
if( sz  > 0.55 ) score += sbig;
else if( sz  > 0.35 ) score += smedium;
else if( sz > 0.15 ) score += ssmall;
scoreboard( "[q][w]turn [o]thrust [p]fire [1]shields" );
}
/* ================================ */
cleanup()
{
int i;
/* erase any old player shots and initialize shots */
Ecolor(Backcolor);
for( i = 0; i < MAXSHOT; i++ ) if( shlife[i] > 0 ) { Emov( oldshx[i], oldshy[i] ); Etext( "+" ); }
Ecolor(Pencolor);
for( i = 0; i < MAXSHOT; i++ ) shlife[i] = 0;
nsh = 0;

/* erase any old killer shots and initialize shots */
Ecolor(Backcolor);
for( i = 0; i < MAXSHOT; i++ ) if( kshlife[i] > 0 ) { Emov( oldkshx[i], oldkshy[i] ); Etext( "*" ); }
Ecolor(Pencolor);
for( i = 0; i < MAXSHOT; i++ ) kshlife[i] = 0;
nksh = 0;

}
/* ================================= */
expshape( x, y, r )
double x, y, r;
{
static int exsx[20] = { 0, 1, 3, 1, 4, 2, 5, 3, 4, 2, 1, 0, -2, -1, -4, -2, -3, -2, -4, -1 };
static int exsy[20] = { 5, 3, 5, 1, 3, 0, -2, -2, -4, -2, -4, -2, -5, -2, -3, -1, 0, 1, 4, 3 }; 

int i;

Emov( x + (exsx[19]*r), y + (exsy[19]*r) );
for( i = 0; i < 20; i++ ) {
	Elin( x + (exsx[i]*r), y + (exsy[i]*r) );
	}
}
/* ================================= */
drawrock( x, y, s )
double x, y, s;
{
register int i;
static int rx[15] = { 2, 3, 4, 5, 4, 3, 3, 0, 0,-3,-2,-4,-2,-2 };
static int ry[15] = { 5, 3, 4, 1,-1,-1,-4,-3,-4,-2,-1, 2, 3, 4 }; 
static double smallx[14], smally[14];
static double medx[14], medy[14];
static double largex[14], largey[14];
static int first = 1;

if( first ) { /* first time thru-- generate the different sizes.. */
	first = 0;
	for( i = 0; i < 14; i++ ) {
		smallx[i] = rx[i] * 0.04;
		smally[i] = ry[i] * 0.04;
		medx[i] = rx[i] * 0.08;
		medy[i] = ry[i] * 0.08;
		largex[i] = rx[i] * 0.12;
		largey[i] = ry[i] * 0.12;
		}
	}

if( s > 0.5 ) {
	Emov( x + largex[13], y + largey[13] );
	for( i = 0; i < 14; i++ ) Elin( x + largex[i], y + largey[i] );
	}
else if( s > 0.3 ) {
	Emov( x + medx[13], y + medy[13] );
	for( i = 0; i < 14; i++ ) Elin( x + medx[i], y + medy[i] );
	}
else 	{
	Emov( x + smallx[13], y + smally[13] );
	for( i = 0; i < 14; i++ ) Elin( x + smallx[i], y + smally[i] );
	}
}
/* ======================================= */
drawstar( x, y, s )
double x, y, s;
{
register int i;
static int rx[15] = { 0, 1, 5, 2, 3, 0, -3, -2, -5, -1 };
static int ry[15] = { 5, 2, 2, 0, -3, -2, -3, 0, 2, 2 };
static double smallx[14], smally[14];
static double medx[14], medy[14];
static double largex[14], largey[14];
static int first = 1;

if( first ) { /* first time thru-- generate the different sizes.. */
	first = 0;
	for( i = 0; i < 10; i++ ) {
		smallx[i] = rx[i] * 0.04;
		smally[i] = ry[i] * 0.04;
		medx[i] = rx[i] * 0.08;
		medy[i] = ry[i] * 0.08;
		largex[i] = rx[i] * 0.12;
		largey[i] = ry[i] * 0.12;
		}
	}

if( s > 0.5 ) {
	Emov( x + largex[9], y + largey[9] );
	for( i = 0; i < 10; i++ ) Elin( x + largex[i], y + largey[i] );
	}
else if( s > 0.3 ) {
	Emov( x + medx[9], y + medy[9] );
	for( i = 0; i < 10; i++ ) Elin( x + medx[i], y + medy[i] );
	}
else 	{
	Emov( x + smallx[9], y + smally[9] );
	for( i = 0; i < 10; i++ ) Elin( x + smallx[i], y + smally[i] );
	}
}


/* ======================================= */
help()
{
double h, x1, y1, x2, y2;
char s[80];

RETRY:
	/* to pause */
	Eclr( ); h = 9.8; 

	Ebackcolor( "gray(0.8)" );
	Ecolor( "black" );
	
	getgui_buttons( "Select one:", "Help,Speed,Quit,OK", s );

	Eflush();
	if( strcmp( s, "Quit" )==0 ) {
		fprintf( stderr, "Quit: score %d\n", score );
		exit(0);
		}
	else if( strcmp( s, "Help" )==0 ) {

		/* getgui_set( "textsize", "8" ); */

getgui_buttons( "This game mimics the controls of the arcade version. \n\
\n\
Place your left hand over the Q and W keys\n\
(Q rotates the spaceship counterclockwise, \n\
and W rotates it clockwise).\n\
\n\
Place your right hand over the O and P keys.\n\
(O is thrust; P fires phaser).\n\
\n\
During play, hit the SPACE BAR to pause the game.\n\
For shield, press RETURN; shield will cease when next\n\
shot is fired.  Shields eventually wear out.\n\
\n\
When ready to start, press the SPACE BAR.\n\
Your spaceship will appear in center screen.\n\
\n\
SCORING:\n\
  Rocks (etc) are worth: big=20 medium=40 small=60.\n\
  There are three types of enemy vessels that may attack you:\n\
  <XXX>  worth 200       <!>   worth 500       <?>   worth 1000\n\
\n\
  EXTRA PLAY is earned for every 10000 points.", "OK", s );

		/* getgui_set( "textsize", "" ); */
		goto RETRY;
		}
	else if( strcmp( s, "Speed" )==0 ) {
		char buf[200];
		long atol();
		sprintf( buf, "%ld", dragfactor/1000 );
		getgui_keyinput( "Enter drag factor \n\
(lower number makes things move faster):", buf, 3, "OK", buf );
		dragfactor = atol( buf ) * 1000;
		goto RETRY;
		}
	/* Emessage( "STAND BY.. [q][w]turn [o]thrust [p]fire [1]shields" );
	 * sleep( 2 );
	 * scoreboard( "[q][w]turn [o]thrust [p]fire [1]shields" );
	 */
	Ebackcolor( Backcolor );
	Ecolor( Pencolor );
	Eclr();
	}
/* ==================================================== */

Emessage( s )
char s[];
{
int oldsize;
Ecblock( 0.0, winsize_y-0.3, winsize_x, winsize_y, Backcolor, 0 );
Emov( 0.3, winsize_y-0.28 );
oldsize = Ecurtextsize;
Etextsize( 12 );
Etext( s );
Etextsize( oldsize );
}

#ifdef CUT
/* ===================================================== */
/* random() returns a "random" number between 0.0 and 1.0 */
/* Auth: Steve Grubb */

double GL_rand()
{
double r;
static int first = 1;
if( first ) {
	srand( getpid() % 1000 );
	first = 0;
	}

/* r = rand() / 2147483647.0; */
r = GL_rand();  /* sys V */
if( r < 0.0 || r > 1.0 ) { printf( "%f: rand return out of range\n", r ); exit(); }
return( r );
}

/* simple event handler (X mode only) */
Ehandle_events( x, y, e )
double x, y;
int e;
{
if( e == E_EXPOSE || e == E_RESIZE ) Erestorewin();
}
#endif
